% -------------------------------------------------------------------------
% THIS SCRIPT IS USED TO FIND THE ENERGY AND MASS OF A BOUND PAIR ON THE 
% BODY-CENTERED CUBIC LATTICE.
% WE DEFINE THE PARAMETERS BELOW FOR THE CALCULATION
% (a) Momentum (along the cartesian coordinates)
% (b) Interaction parameters: U, V and V' (+ve = repulsive, -ve =
% attractive
% (c) hopping parameters: t and t' (t' is generally a multiple of t)
%
% AFTER THE DECLARATION OF THE VARIABLES ABOVE, THE ORDER OF
% EXECUTION OF THE SCRIPT IS SUMMARISED AS FOLLOWS:
% (i) ENERGY: RUN A BINARY SEARCH TO FIND THE ENERGY OF THE BOUND PARTICLES.
% THIS IS ACHIEVED WHEN THE EIGNEVALUE OF THE LOWEST ENERGY IS EQUAL TO 
% UNITY (1) i.e. THE EIGENVALUE OF EQN (A23) - SEE APPENDIX A OF OUR PAPER.
%(ii) MASS: FIND THE ENERGY (STEP i ABOVE) BOTH FOR ZERO MOMENTUM AND A SMALL
% PAIR MOMENTUM. THE INVERSE MASS IS PROPORTIONAL TO THE SECOND DERIVATIVE OF THE ENERGY
% THUS, WE CAN USE THE FORWARD DIFFERENCE FORMULA
% (1/Mass) ~ D^2E / DK^2 = [ E(K) - 2 E(0) + E(-K) ] / 2K^2
% Note that E(k) = E(-k), HENCE  
% (1 / Mass) ~ [E(K) - E(0)] /K^2
%
%
% AUTHOR: Ganiyu Adebanjo, 2022
% -------------------------------------------------------------------------

clear all
clearvars
clc

% entry to MAIN program
tic;
format long
format compact

arg_list = argv()

if (length(arg_list)<4)
  disp("ERROR insuffient arguments")
  exit
end

outputFileNameComponents = {'resultsBCC',num2str(str2double(arg_list(1))),num2str(str2double(arg_list(2))),num2str(str2double(arg_list(3))),num2str(str2double(arg_list(4))),'.dat'}

outputFileName = strjoin(outputFileNameComponents,"_")

				% syms pi;

MomentumX = [0.0125 0]*pi; %linspace(0*pi,0*pi,21);
MomentumY = [0 0]*pi; %linspace(1*pi,0*pi,21); 
MomentumZ = [0 0]*pi; %linspace(1*pi,0*pi,21);

u = str2double(arg_list(1)) % Onsite interaction
v = str2double(arg_list(2)) % NN interaction
v_prime = str2double(arg_list(3)) % NN interaction
t_prime = str2double(arg_list(4)) % NN interaction


eigindex = 8;
%eigindex = floor(str2double(arg_list(5)))


t=1; % NN hopping parameter


% TEST RADIUS CALCULATION - SHOULD COMMENT OUT
%[radiusCalculated] = calculateRadiusBCC(t,t_prime,u,v,v_prime,0,0,0,-50.0);
%disp(radiusCalculated)


					   noninteractingpairenergy = -16.*t-12.*t_prime
					   
% FIRST DETERMINE THE ENERGY

for momentum = 1:length(MomentumZ)
  
  p_z = MomentumZ(momentum);
  p_x = MomentumX(momentum);
  p_y = MomentumY(momentum);  
  
  for lu = 1:length(u)
    u_select = u(lu);
    
    for lv = 1:length(v)
      v_select = v(lv);
      
      tol = 1e-7; % tolerance used for convergence
      
		    % the upper and lower limits should be wide enough
					   
					   lower_bound_energy = -abs(4.*(v_select+v_prime))-16.*t-12.*t_prime;
      
	       % change - the bounds were previously wrong - above the
               % range for the free pair energy
      
      upper_bound_energy = noninteractingpairenergy;
      
      mid_point_energy = (lower_bound_energy + upper_bound_energy)./2.0;
      
      for i=eigindex:eigindex % we are only interested in the lowest bound state
        
	[lowerE, upperE, eigvalret] = check_root_intervalBCC(i,upper_bound_energy,lower_bound_energy,mid_point_energy, ...
							     u_select,v_select,v_prime,t,t_prime,tol,p_x,p_y,p_z);
        
	if (lowerE ~=0 && upperE ~= 0)
				%EminDetect = upperE
				%EmaxDetect = lowerE
%resL = searchSingletEigenvalueBCC(upperE,lowerE,u_select,v_select,v_prime,t,t_prime,tol,p_x,p_y,p_z);
	  resL = (lowerE + upperE )./ 2.0;
          finalEnergyArray(i)= resL;
          
        end
	
      end
      
    end
    
  end
  
	  % Here : is shorthand for 1:end. Really unsure why there are
	  % two : here though
  
  saveEs(:,:,momentum) = [finalEnergyArray(eigindex) u_select v_select v_prime t t_prime p_x/pi p_y/pi p_z/pi]; % print result to screen
  
  energyMomentum(momentum) = resL;
  eigenvalMomentum(momentum) = eigvalret;
  
end

saveEs % show the whole array as unclear why pulling certain values out of the array here

bareinvmass = t+t_prime
deltaenergy = (energyMomentum(1)- energyMomentum(2))
deltamomentum = (MomentumX(2)-MomentumX(1))
invmass = deltaenergy ./ deltamomentum.^2

		% the bare mass scales as t + t', hence the multiplier
mass = bareinvmass./invmass;

[saveEs(1) saveEs(eigindex) mass] % print to screen

			   % one can save these variables to file e.g.
			   %  save('filename.mat','saveEs','mass')


				% NOW DETERMINE THE RADIUS

% SMALL BUG HERE - NEED relL FOR THE MOMENTUM = 0 CASE, THIS IS FOR NON-ZERO MOMENTUM BUT LABELLED AS ZERO
% FIXED BY REARRANGING THE ORDER OF THE MOMENTUM LOOP

[radiusCalculated] = calculateRadiusBCC(t,t_prime,u,v,v_prime,0,0,0,energyMomentum(2),eigindex);


fileID = fopen(outputFileName,'w');

fprintf(fileID,"# mass u v v_p t t_p\n");

fprintf(fileID,"%f %f %f %f %f %.11f %.11f %.11f %.11f %f %f %f %.11f %.11f\n",u,v,v_prime,t,t_prime,mass,energyMomentum(2),energyMomentum(1),radiusCalculated,noninteractingpairenergy,bareinvmass,deltamomentum,eigenvalMomentum(2),eigenvalMomentum(1))  

fclose(fileID);

toc % prints run time at the end of MAIN program run

