% -------------------------------------------------------------------------
% THIS SCRIPT IS USED TO FIND THE ENERGY AND MASS OF A BOUND PAIR ON THE 
% SIMPLE CUBIC LATTICE.
% WE DEFINE THE PARAMETERS BELOW FOR THE CALCULATION
% (a) Momentum (along the cartesian coordinates)
% (b) Interaction parameters: U, V and V' (+ve = repulsive, -ve =
% attractive)
% (c) hopping parameters: t and t' (t' is chosen as a multiple of t)
%
% AFTER THE DECLARATION OF THE VARIABLES ABOVE, THE ORDER OF
% EXECUTION OF THE SCRIPT IS SUMMARISED AS FOLLOWS:
% (i) ENERGY: RUN A BINARY SEARCH TO FIND THE ENERGY OF THE BOUND PARTICLES.
% THIS IS ACHIEVED WHEN THE EIGNEVALUE OF THE LOWEST ENERGY IS EQUAL TO 
% UNITY (1) i.e. THE EIGENVALUE OF EQN (A18) - SEE APPENDIX A OF OUR PAPER.
% PAIR MOMENTUM. THE INVERSE MASS IS PROPORTIONAL TO THE SECOND DERIVATIVE OF THE ENERGY
% THUS, WE CAN USE THE FORWARD DIFFERENCE FORMULA
% (1/Mass) ~ D^2E / DK^2 = [ E(K) - 2 E(0) + E(-K) ] / 2K^2
% Note that E(k) = E(-k), HENCE  
% (1 / Mass) ~ [E(K) - E(0)] /K^2
%
%
% AUTHOR: Ganiyu Adebanjo, 2022
% UPDATES (primarily for efficiency): Jim Hague, 2023
% -------------------------------------------------------------------------

clear all
clearvars
clc

% entry to MAIN 
tic;
format long
format compact

arg_list = argv()

if (length(arg_list)<4)
  disp("ERROR insuffient arguments")
  exit
end


outputFileNameComponents = {'resultsSC',num2str(str2double(arg_list(1))),num2str(str2double(arg_list(2))),num2str(str2double(arg_list(3))),num2str(str2double(arg_list(4))),'.dat'}

outputFileName = strjoin(outputFileNameComponents,"_")


% syms pi;

MomentumX = [0.0125 0]*pi; %linspace(0*pi,0*pi,21);
MomentumY = [0 0]*pi;
MomentumZ = [0 0]*pi;


u = str2double(arg_list(1)) % Onsite interaction
v = str2double(arg_list(2)) % NN interaction
v_prime = str2double(arg_list(3)) % NN interaction
t_prime = str2double(arg_list(4)) % NN interaction

t=1; % NN hopping parameter

noninteractingenergy = -12.*t-24.*t_prime;

eigindex = 10;
% eigindex = floor(str2double(arg_list(5)))

for momentum = 1:length(MomentumX)
  
  p_x = MomentumX(momentum)
  p_y = MomentumY(momentum)
  p_z = MomentumZ(momentum)
  
  
				%u = 10000; % Onsite interaction
				%v = -1000; % NN interaction
  
				%t=1; % NN hopping parameter
%t_prime = 0.85*t; % NN hopping parameter to be changed accordingly to x*t where 0<=x<=1
  
  
  for lu = 1:length(u) % incase we are using an array of onsite interaction
    u_select = u(lu);
    
    for lv = 1:length(v) % incase we are using an array of NN interaction
      v_select = v(lv);
		  % v_prime = v_select; % can be set as y*v where y>=0
      
      tol = 1e-7; % tolerance used for convergence
      
		    % the upper and lower limits should be wide enough
		    %lower_bound_energy = -1215.19;
lower_bound_energy = -abs(4*(v_select+v_prime))+noninteractingenergy;
      upper_bound_energy = noninteractingenergy;
      mid_point_energy = (lower_bound_energy + upper_bound_energy)/2.0;
      
      for i=eigindex:eigindex % we are only interested in the lowest bound state
        
	      [lowerE, upperE, eigvalret] = check_root_interval(i,upper_bound_energy,lower_bound_energy,mid_point_energy, ...
                                               u_select,v_select,v_prime,t,t_prime,tol,p_x,p_y,p_z);
        
        if (lowerE ~=0 && upperE ~= 0)
				%EminDetect = upperE
				%EmaxDetect = lowerE
%resL = searchSingletEigenvalue(upperE,lowerE,u_select,v_select,v_prime,t,t_prime,tol,p_x,p_y,p_z);
resL = (lowerE + upperE) ./ 2.0;
          finalEnergyArray(i)= resL;
energyMomentum(momentum) = resL;
eigvalMomentum(momentum) = eigvalret;
        end
      end
      
    end
    
    
  end
  
  
  
  saveEs(:,:,momentum) = [finalEnergyArray(eigindex) u_select v_select v_prime t t_prime p_x/pi p_y/pi p_z/pi]; % print result to screen
  
end

bareinvmass = t + 4.*t_prime
deltaenergy = (energyMomentum(1)- energyMomentum(2))
deltamomentum = (MomentumX(2)-MomentumX(1))
invmass = deltaenergy./deltamomentum^2

	       % the bare mass scales as t + 4t', hence the multiplier
mass = bareinvmass./invmass

[radiusCalculated] = calculateRadiusSC(t,t_prime,u,v,v_prime,0,0,0,resL,eigindex)

fileID = fopen(outputFileName,'w');

fprintf(fileID,"# mass u v v_p t t_p\n");
	  

fprintf(fileID,"%f %f %f %f %f %.11f %.11f %.11f %.11f %.11f %f %f %f %11.f %11.f %11.f %11.f\n",u,v,v_prime,t,t_prime,mass,energyMomentum(2),energyMomentum(1),resL,radiusCalculated,noninteractingenergy,bareinvmass,deltamomentum,real(eigvalMomentum(2)),real(eigvalMomentum(1)),imag(eigvalMomentum(2)),imag(eigvalMomentum(1)))  

fclose(fileID);


toc % prints run time at the end of MAIN program run

  
