% -------------------------------------------------------------------------
% THIS SCRIPT IS USED TO FIND THE ENERGY AND MASS OF A BOUND PAIR ON THE 
% SQUARE LATTICE.
% WE DEFINE THE PARAMETERS BELOW FOR THE CALCULATION
% (a) Momentum (along the cartesian coordinates)
% (b) Interaction parameters: U, V and V' (+ve = repulsive, -ve =
% attractive)
% (c) hopping parameters: t and t' (t' is chosen as a multiple of t)
%
% AFTER THE DECLARATION OF THE VARIABLES ABOVE, THE ORDER OF
% EXECUTION OF THE SCRIPT IS SUMMARISED AS FOLLOWS:
% (i) ENERGY: RUN A BINARY SEARCH TO FIND THE ENERGY OF THE BOUND PARTICLES.
% THIS IS ACHIEVED WHEN THE EIGNEVALUE OF THE LOWEST ENERGY IS EQUAL TO 
% UNITY (1) i.e. THE EIGENVALUE OF EQN (A13) - SEE APPENDIX A OF OUR PAPER.
% PAIR MOMENTUM. THE INVERSE MASS IS PROPORTIONAL TO THE SECOND DERIVATIVE OF THE ENERGY
% THUS, WE CAN USE THE FORWARD DIFFERENCE FORMULA
% (1/Mass) ~ D^2E / DK^2 = [ E(K) - 2 E(0) + E(-K) ] / 2K^2
% Note that E(k) = E(-k), HENCE  
% (1 / Mass) ~ [E(K) - E(0)] /K^2
%
%
% AUTHOR: Ganiyu Adebanjo, 2022
% -------------------------------------------------------------------------


clear all
clearvars
clc

				% entry to MAIN program
tic;
format long
format compact

arg_list = argv()

if (length(arg_list)<4)
  disp("ERROR insuffient arguments")
  exit
end

				% syms pi;

outputFileNameComponents = {'resultsSquare',num2str(str2double(arg_list(1))),num2str(str2double(arg_list(2))),num2str(str2double(arg_list(3))),num2str(str2double(arg_list(4))),'.dat'}

outputFileName = strjoin(outputFileNameComponents,"_")

MomentumX = [0.0125 0 0]*pi; %linspace(0*pi,0*pi,21);
MomentumY = [0 0 0]*pi;
MomentumZ = [0 0 0.0125]*pi;

t_perp = 0.04

for momentum = 1:length(MomentumX)
  
  p_x = MomentumX(momentum)
  p_y = MomentumY(momentum)
  p_z = MomentumZ(momentum)
  
  
  u = str2double(arg_list(1)); % Onsite interaction
  v = str2double(arg_list(2)) % NN interaction
  
  t=1 % NN hopping parameter
  t_prime = str2double(arg_list(4))*t % NNN hopping parameter, x*t where 0<=x<=1
  
  
  for lu = 1:length(u) % incase of array of onsite interaction
    u_select = u(lu);
    
    for lv = 1:length(v) % incase of an array NN interaction
      v_select = v(lv);
      v_prime = str2double(arg_list(3)); % can be set to multiple of v i.e. y*v_select where y>=0
      
      tol = 1e-9; % tolerance used for convergence
      
		    % the upper and lower limits should be wide enough
		    lower_bound_energy = -abs(4.0*v_select)-abs(4.0.*v_prime)-8.0.*t - 8.0.*t_prime - 4.0.*t_perp;
      upper_bound_energy = -8.0.*t - 8.0.*t_prime - 4.0.*t_perp;
      mid_point_energy = (lower_bound_energy + upper_bound_energy)/2.0;
      
      for i=5:5 % we are only interested in the lowest bound state
        
        [lowerE, upperE] = check_root_intervalSq(i,upper_bound_energy,lower_bound_energy,mid_point_energy, ...
						 u_select,v_select,v_prime,t,t_prime,t_perp,tol,p_x,p_y,p_z);
        
        if (lowerE ~=0 && upperE ~= 0)
				%EminDetect = upperE
				%EmaxDetect = lowerE
%resL = searchSingletEigenvalueSq(upperE,lowerE,u_select,v_select,v_prime,t,t_prime,tol,p_x,p_y);
	  resL = (upperE + lowerE)./2.0;
          finalEnergyArray(i)= resL;
          
        end
      end
      
    end
    
    
  end
  
	energyMomentum(momentum) = resL;
  
  saveEs(:,:,momentum) = [finalEnergyArray(5) u_select v_select v_prime t t_prime p_x/pi p_y/pi]; % print result to screen
  
end

	       % the bare mass scales as t + 2t', hence the multiplier
    mass = (1/((energyMomentum(1)- energyMomentum(2))/(MomentumX(2)-MomentumX(1))^2)).*(t+(2.*t_prime))
				%  save('name.mat','saveEs','mass')

    massz = (1/((energyMomentum(3)- energyMomentum(2))/(MomentumZ(2)-MomentumZ(3))^2)).*(t_perp)

    [radiusCalculated, radiusx, radiusy, radiusz] = calculateRadiusSquare(t,t_prime,t_perp,u,v,v_prime,0,0,0,energyMomentum(2),5)

fileID = fopen(outputFileName,'w');

fprintf(fileID,"# mass u v v_p t t_p\n");
    
fprintf(fileID,"%f %f %f %f %f %.11f %.11f %.11f %.11f %.11f %.11f %.11f %.11f %.11f\n",u,v,v_prime,t,t_prime,mass,massz,energyMomentum(2),energyMomentum(1),energyMomentum(3),radiusCalculated,radiusx,radiusy,radiusz)  

fclose(fileID);



toc % prints run time at the end of MAIN program run


